-- https://github.com/Alumian/CEPGP-Retail/ all credits.

local function HexToRGBPerc(hex)
	local rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
end

local dkp_color_0, dkp_color_1, dkp_color_2 = HexToRGBPerc("e74c3c")

local function CEPGP_itemExists(id)
	if not id or not tonumber(id) then return false; end
	if C_Item.DoesItemExistByID(tonumber(id)) then
		return true;
	else
		return false;
	end
end
		
local CEPGP_tokens = {
	Heads = {
		["Trinket"] = {
			[18423] = 74,	-- of Onyxia (Alliance) -- Can also be a neck and ring
			[18422] = 74,	-- of Onyxia (Horde) -- Same deal
			[19802] = 68,	-- Heart of Hakkar
			[22520] = 90	-- Phylactery of Kel'Thuzad
		},
		["Neck"] = {
			[21220] = 70,	-- of Ossirian the Unscarred
			[21221] = 88 	-- Eye of C'Thun -- Can also be a cloak or ring
		},
		["Holdable"] = {
			[19003] = 83, 	-- of Nefarian (Alliance) -- Can also be neck and ring
			[19002] = 83	-- of Nefarian (Horde) -- Same deal
		}
	},

	ZG = {
		["Wrist"] = {
			[19717] = 61,	-- Armsplint
			[19716] = 61,	-- Bindings
			[19718] = 61,	-- Stanchion
		},
		["Waist"] = {
			[19719] = 61,	-- Girdle
			[19720] = 61	-- Sash
		},
		["Chest"] = {
			[19724] = 65,	-- Aegis
			[19723] = 65,	-- Kossack
			[19722] = 65	-- Tabard
		},
		["Shoulder"] = {
			[19721] = 68	-- Shawl
		}
	},

	AQ = {
		["Cloak"] = {
			[20885] = 67,	-- Martial Drake
			[20889] = 67	-- Regal Drape
		},
		["Finger"] = {
			[20888] = 65,	-- Ceremonial Ring
			[20884] = 65	-- Magisterial Ring
		},
		["Weaponoffhand"] = {
			[20886] = 70,	-- Spiked Hilt -- Exceptions apply - Paladin / Shaman weapon are main hand
			[21232] = 79,	-- Imperial Qiraji Armaments -- Can also be a ranged weapon or shield
		},
		["Weaponmainhand"] = {
			[20890] = 70	-- Ornate Hilt
		},
		["TwoHweapon"] = {
			[21237] = 79	-- Imperial Qiraji Regalia -- Can also be a one-handed weapon
		},
		["Feet"] = {
			[20928] = 78,	-- Qiraji Bindings of Command -- Can also be shoulders
			[20932] = 78	-- Qiraji Bindings of Dominance -- same deal
		},
		["Chest"] = {
			[20933] = 88,	-- Husk of the Old God
			[20929] = 88	-- Carapace of the Old God
		},
		["Legs"] = {
			[20927] = 81,	-- Ouro's Intact Hide
			[20931] = 81	-- Skin of the Great Sandworm
		}
	},

	T3 = {
		["Shoulder"] = {
			[22368] = 86,	-- Shoulderpads
			[22354] = 86,	-- Pauldrons
			[22361] = 86	-- Spaulders
		},
		["Feet"] = {
			[22372] = 86,	-- Sandals
			[22365] = 86,	-- Boots
			[22358] = 86	-- Sabatons
		},
		["Wrist"] = {
			[22369] = 88,	-- Bindings
			[22362] = 88,	-- Wristguards
			[22355] = 88	-- Bracers
		},
		["Hand"] = {
			[22357] = 88,	-- Gauntlets
			[22364] = 88,	-- Handguards
			[22371] = 88,	-- Gloves
		},
		["Waist"] = {
			[22363] = 88,	-- Girdle
			[22370] = 88,	-- Belt
			[22356] = 88	-- Waistguard
		},
		["Legs"] = {
			[22359] = 88,	-- Legguards
			[22352] = 88,	-- Legplates
			[22366] = 88	-- Leggings
		},
		["Head"] = {
			[22367] = 88,	-- Circlet
			[22360] = 88,	-- Headpiece
			[22353] = 88	-- Helmet
		},
		["Chest"] = {
			[22350] = 92,	-- Tunic
			[22351] = 92,	-- Robe
			[22349] = 92	-- Breastplate
		}
	}
}

local COEF = 4.83;
local MOD = 0.5;
local MOD_COEF = 2;
local OVERRIDE_INDEX = {};
local SLOTWEIGHTS = {
	["2HWEAPON"] = 2,
	["WEAPONMAINHAND"] = 1.5,
	["WEAPON"] = 1.5,
	["WEAPONOFFHAND"] = 0.5,
	["HOLDABLE"] = 0.5,
	["SHIELD"] = 0.5,
	["RANGED"] = 0.5,
	["RANGEDRIGHT"] = 0.5,
	["THROWN"] = 0.5,
	["RELIC"] = 0.5,
	["HEAD"] = 1,
	["NECK"] = 0.5,
	["SHOULDER"] = 0.75,
	["CLOAK"] = 0.5,
	["CHEST"] = 1,
	["ROBE"] = 1,
	["WRIST"] = 0.5,
	["HAND"] = 0.75,
	["WAIST"] = 0.75,
	["LEGS"] = 1,
	["FEET"] = 0.75,
	["FINGER"] = 0.5,
	["TRINKET"] = 0.75,
	["EXCEPTION"] = 1
}

function CEPGP_calcGP(link, quantity, id)

	local name, rarity, ilvl, itemType, subType, slot, classID, subClassID;
	if id then
		name, link, rarity, ilvl, itemType, subType, _, _, slot, _, _, classID, subClassID = GetItemInfo(id);
	elseif link then
		name, _, rarity, ilvl, itemType, subType, _, _, slot, _, _, classID, subClassID = GetItemInfo(link);
	else
		return 0;
	end
	if not name and CEPGP_itemExists(tonumber(id)) then
		local item = Item:CreateFromItemID(tonumber(id));
		item:ContinueOnItemLoad(function()
			name, link, rarity, ilvl, itemType, subType, _, _, slot, _, _, classID, subClassID = GetItemInfo(id);
			for k, v in pairs(OVERRIDE_INDEX) do
				if k == link then
					return v;
				end
				local temp = string.lower(string.gsub(name, " ", ""));
				k = string.lower(string.gsub(k, " ", ""));
				if string.lower(temp) == string.lower(k) then
					return v;
				end
			end
			
			for _, k in pairs(CEPGP_tokens) do
			for slotName, v in pairs(k) do
				if k[slotName][tonumber(id)] then
					slot = "INVTYPE_" .. string.upper(slotName);
					ilvl = k[slotName][tonumber(id)];
					break;
				end
			end
		end
		if slot == "" or slot == nil then
			slot = "INVTYPE_EXCEPTION";
		end
			
			if slot == "INVTYPE_ROBE" then slot = "INVTYPE_CHEST"; end
			if slot == "INVTYPE_WEAPON" then slot = "INVTYPE_WEAPONOFFHAND"; end
			slot = strsub(slot,strfind(slot,"INVTYPE_")+8,string.len(slot));
			slot = SLOTWEIGHTS[slot];
			if ilvl and rarity and slot then
				return math.floor((((COEF * (MOD_COEF^((ilvl/26) + (rarity-4))) * slot)*MOD)*quantity));
			else
				return 0;
			end
		end);
	else
		if not ilvl then ilvl = 0; end
		for k, v in pairs(OVERRIDE_INDEX) do
			if k == link then
				return v;
			end
			local temp = string.lower(string.gsub(name, " ", ""));
			k = string.lower(string.gsub(k, " ", ""));
			if string.lower(temp) == string.lower(k) then
				return v;
			end
		end
		for _, k in pairs(CEPGP_tokens) do
			for slotName, v in pairs(k) do
				if k[slotName][tonumber(id)] then
					slot = "INVTYPE_" .. string.upper(slotName);
					ilvl = k[slotName][tonumber(id)];
					break;
				end
			end
		end
		if slot == "" or slot == nil then
			slot = "INVTYPE_EXCEPTION";
		end
		
		if slot == "INVTYPE_ROBE" then slot = "INVTYPE_CHEST"; end
		if slot == "INVTYPE_WEAPON" then slot = "INVTYPE_WEAPONOFFHAND"; end
		slot = strsub(slot,strfind(slot,"INVTYPE_")+8,string.len(slot));
		slot = SLOTWEIGHTS[slot];
		if ilvl and rarity and slot then
			return math.floor((((COEF * (MOD_COEF^((ilvl/26) + (rarity-4))) * slot)*MOD)*quantity));
		else
			return 0;
		end
	end
end

local function DKPTooltip(tooltip)
	local itemTName, itemTLink = tooltip:GetItem();

	GameTooltip:AddLine("DKP: "..CEPGP_calcGP(itemTLink, 1), dkp_color_0, dkp_color_1, dkp_color_2, true);
end
GameTooltip:HookScript("OnTooltipSetItem", DKPTooltip)
